/*
	file:
		IDirect3D9Hook.cpp
	desc:
		This is the code for wrapping the IDirect3D9 interface.
*/

// Includes
#include "IDirect3D9Hook.h"
#include "Custom.h"

// Externals
extern bool bWindowedMode;
extern IDirect3D9 *pD3D;
extern IDirect3DDevice9 *pD3DDevice;
extern IDirect3DDevice9Hook *pD3DDeviceHook;

// Globals
D3DDISPLAYMODE D3DDisplayMode;

// Functions
HRESULT __stdcall IDirect3D9Hook::CreateDevice(UINT Adapter, D3DDEVTYPE DeviceType, HWND hFocusWindow, DWORD BehaviorFlags, D3DPRESENT_PARAMETERS* pPresentationParameters, IDirect3DDevice9** ppReturnedDeviceInterface)
{
	HRESULT hr = pD3D->CreateDevice(Adapter, DeviceType, hFocusWindow, BehaviorFlags, pPresentationParameters, ppReturnedDeviceInterface);
	if (SUCCEEDED(hr))
	{
		pD3DDevice = *ppReturnedDeviceInterface;
		pD3DDeviceHook = &D3DDevice9Hook;
		CustomInitialize(pD3DDevice, pPresentationParameters);
		*ppReturnedDeviceInterface = pD3DDeviceHook;
	}
	else
		*ppReturnedDeviceInterface = NULL;
	return hr;
}

ULONG __stdcall IDirect3D9Hook::AddRef(void)
{
	if (++m_iRef > 0)
	{
		return pD3D->AddRef();
	}
	else
	{
		return 0;
	}
}

ULONG __stdcall IDirect3D9Hook::Release(void)
{
	if (--m_iRef > 0)
	{
		return pD3D->Release();
	}
	else
	{
		return 0;
	}
}

//################################################################################

HRESULT __stdcall IDirect3D9Hook::QueryInterface(REFIID iid, void ** ppvObject)
{
	return pD3D->QueryInterface(iid, ppvObject);
}

HRESULT __stdcall IDirect3D9Hook::RegisterSoftwareDevice(void *pInitializeFunction)
{
	return pD3D->RegisterSoftwareDevice(pInitializeFunction);
}

UINT __stdcall IDirect3D9Hook::GetAdapterCount(void)
{
	return pD3D->GetAdapterCount();
}

HRESULT __stdcall IDirect3D9Hook::GetAdapterIdentifier(UINT Adapter, DWORD Flags, D3DADAPTER_IDENTIFIER9* pIdentifier)
{
	return pD3D->GetAdapterIdentifier(Adapter, Flags , pIdentifier);;
}

UINT __stdcall IDirect3D9Hook::GetAdapterModeCount(UINT Adapter,D3DFORMAT Format)
{
	return pD3D->GetAdapterModeCount(Adapter, Format);
}

HRESULT __stdcall IDirect3D9Hook::EnumAdapterModes(UINT Adapter, D3DFORMAT Format, UINT Mode, D3DDISPLAYMODE* pMode)
{
	return pD3D->EnumAdapterModes(Adapter, Format, Mode, pMode);
}

HRESULT __stdcall IDirect3D9Hook::GetAdapterDisplayMode(UINT Adapter,D3DDISPLAYMODE *pMode)
{
	return pD3D->GetAdapterDisplayMode(Adapter, pMode);
}

HRESULT __stdcall IDirect3D9Hook::CheckDeviceType(UINT Adapter, D3DDEVTYPE DevType, D3DFORMAT AdapterFormat, D3DFORMAT BackBufferFormat, BOOL bWindowed)
{
	return pD3D->CheckDeviceType(Adapter, DevType, AdapterFormat, BackBufferFormat, bWindowed);
}

HRESULT __stdcall IDirect3D9Hook::CheckDeviceFormat(UINT Adapter, D3DDEVTYPE DeviceType, D3DFORMAT AdapterFormat, DWORD Usage, D3DRESOURCETYPE RType, D3DFORMAT CheckFormat)
{
	return pD3D->CheckDeviceFormat(Adapter, DeviceType, AdapterFormat, Usage, RType, CheckFormat);
}

HRESULT __stdcall IDirect3D9Hook::CheckDeviceMultiSampleType(UINT Adapter, D3DDEVTYPE DeviceType, D3DFORMAT SurfaceFormat, BOOL Windowed, D3DMULTISAMPLE_TYPE MultiSampleType, DWORD* pQualityLevels)
{
	return pD3D->CheckDeviceMultiSampleType(Adapter, DeviceType, SurfaceFormat, Windowed, MultiSampleType, pQualityLevels);
}

HRESULT __stdcall IDirect3D9Hook::CheckDepthStencilMatch(UINT Adapter, D3DDEVTYPE DeviceType, D3DFORMAT AdapterFormat, D3DFORMAT RenderTargetFormat, D3DFORMAT DepthStencilFormat)
{
	return pD3D->CheckDepthStencilMatch(Adapter, DeviceType, AdapterFormat, RenderTargetFormat, DepthStencilFormat);
}

HRESULT __stdcall IDirect3D9Hook::CheckDeviceFormatConversion(UINT Adapter, D3DDEVTYPE DeviceType, D3DFORMAT SourceFormat, D3DFORMAT TargetFormat)
{
	return pD3D->CheckDeviceFormatConversion(Adapter, DeviceType, SourceFormat, TargetFormat);
}

HRESULT __stdcall IDirect3D9Hook::GetDeviceCaps(UINT Adapter, D3DDEVTYPE DeviceType, D3DCAPS9* pCaps)
{
	return pD3D->GetDeviceCaps(Adapter, DeviceType, pCaps);
}

HMONITOR __stdcall IDirect3D9Hook::GetAdapterMonitor(UINT Adapter)
{
	return pD3D->GetAdapterMonitor(Adapter);
}
