/*
	file:
		dx9hook.cpp
	desc:
		This file contains the dll entry point and initialization code.
	notes:
		Original d3d9.dll exports:
			0000	00025db0	CheckFullscreen
			0004	0005f9b0	D3DPERF_BeginEvent
			0005	000a81a0	D3DPERF_EndEvent
			0006	0001d030	D3DPERF_GetStatus
			0007	00026a30	D3DPERF_QueryRepeatFrame
			0008	00026a90	D3DPERF_SetMarker
			0009	00026c70	D3DPERF_SetOptions
			000a	00026bb0	D3DPERF_SetRegion
			000b	00026af0	DebugSetLevel
			000c	00026c10	DebugSetMute
			000d	00026b50	Direct3DCreate9
			0001	0002b030	Direct3DShaderValidatorCreate9
			0002	0002afd0	PSGPError
			0003	0002aed0	PSGPSampleTexture
*/

// Includes
#include "dx9hook.h"
#include "Custom.h"

HMODULE* modules = NULL;
HINSTANCE hLib;
int moduleCount = 0;

// Functions
BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD fdwReason, LPVOID)	// Entry point.
{
	switch (fdwReason)
	{
		case DLL_PROCESS_ATTACH:
		{
			// Save the DLL instance for later.
			hInst = hInstance;

			// Load the original d3d9.dll
			char strOrigDll[MAX_PATH];
			GetSystemDirectory (strOrigDll, sizeof(strOrigDll));
			strcat(strOrigDll, "\\d3d9.dll");
			hOrigDll = LoadLibrary(strOrigDll);

			// Get function addresses for all d3d8.dll exports
			OldCheckFullscreen = GetProcAddress(hOrigDll, "CheckFullscreen");
			OldD3DPERF_BeginEvent = GetProcAddress(hOrigDll, "D3DPERF_BeginEvent");
			OldD3DPERF_EndEvent = GetProcAddress(hOrigDll, "D3DPERF_EndEvent");
			OldD3DPERF_GetStatus = GetProcAddress(hOrigDll, "D3DPERF_GetStatus");
			OldD3DPERF_QueryRepeatFrame = GetProcAddress(hOrigDll, "D3DPERF_QueryRepeatFrame");
			OldD3DPERF_SetMarker = GetProcAddress(hOrigDll, "D3DPERF_SetMarker");
			OldD3DPERF_SetOptions = GetProcAddress(hOrigDll, "D3DPERF_SetOptions");
			OldD3DPERF_SetRegion = GetProcAddress(hOrigDll, "D3DPERF_SetRegion");
			OldDebugSetLevel = GetProcAddress(hOrigDll, "DebugSetLevel");
			OldDebugSetMute = GetProcAddress(hOrigDll, "DebugSetMute");
			OldDirect3DShaderValidatorCreate9 = GetProcAddress(hOrigDll, "Direct3DShaderValidatorCreate9");
			OldPSGPError = GetProcAddress(hOrigDll, "PSGPError");
			OldPSGPSampleTexture = GetProcAddress(hOrigDll, "PSGPSampleTexture");
			OldDirect3DCreate9 = (Direct3DCreate9_t) GetProcAddress(hOrigDll, "Direct3DCreate9");

			bWindowedMode = false;				// Windowed mode off by default

			//begin sagfx

			//end sagfx

			break;
		}
		case DLL_PROCESS_DETACH:
		{
			if (hOrigDll)	// If the original dll is loaded...
			{
				CustomFree();
				FreeLibrary(hOrigDll);	// Unload it.
				hOrigDll = NULL;
			}
			break;
		}
	}
	return true;
}

void __declspec(naked) NewCheckFullscreen(void)
{
	__asm jmp OldCheckFullscreen;
}

void __declspec(naked) NewD3DPERF_BeginEvent(void)
{
	__asm jmp OldD3DPERF_BeginEvent;
}

void __declspec(naked) NewD3DPERF_EndEvent(void)
{
	__asm jmp OldD3DPERF_EndEvent;
}

void __declspec(naked) NewD3DPERF_GetStatus(void)
{
	__asm jmp OldD3DPERF_GetStatus;
}

void __declspec(naked) NewD3DPERF_QueryRepeatFrame(void)
{
	__asm jmp OldD3DPERF_QueryRepeatFrame;
}

void __declspec(naked) NewD3DPERF_SetMarker(void)
{
	__asm jmp OldD3DPERF_SetMarker;
}

void __declspec(naked) NewD3DPERF_SetOptions(void)
{
	__asm jmp OldD3DPERF_SetOptions;
}

void __declspec(naked) NewD3DPERF_SetRegion(void)
{
	__asm jmp OldD3DPERF_SetRegion;
}

void __declspec(naked) NewDebugSetLevel(void)
{
	__asm jmp OldDebugSetLevel;
}

void __declspec(naked) NewDebugSetMute(void)
{
	__asm jmp OldDebugSetMute;
}

void __declspec(naked) NewDirect3DShaderValidatorCreate9(void)
{
	__asm jmp OldDirect3DShaderValidatorCreate9;
}

void __declspec(naked) NewPSGPError(void)
{
	__asm jmp OldPSGPError;
}

void __declspec(naked) NewPSGPSampleTexture(void)
{
	__asm jmp OldPSGPSampleTexture;
}

IDirect3D9* WINAPI NewDirect3DCreate9(UINT SDKVersion)
{
	pD3D = OldDirect3DCreate9(SDKVersion);	// Create a real IDirect3D9 interface for us.

	pD3DHook = &D3D9Hook;	// Get a pointer to our IDirect3D9Hook interface.
	return pD3DHook;		// Give San Andreas our fake IDirect3D9 interface pointer.
}
